import 'dart:convert';

class GeoCoordinates {
  double longitude;
  double latitude;
  double? altitude;
  double? timestamp;

  GeoCoordinates({
    required this.longitude,
    required this.latitude,
    this.altitude,
    this.timestamp,
  });

  GeoCoordinates copyWith({
    double? longitude,
    double? latitude,
    double? altitude,
    double? timestamp,
  }) {
    return GeoCoordinates(
      longitude: longitude ?? this.longitude,
      latitude: latitude ?? this.latitude,
      altitude: altitude ?? this.altitude,
      timestamp: timestamp ?? this.timestamp,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'Longitude': longitude,
      'Latitude': latitude,
      'Altitude': altitude,
      'Timestamp': timestamp,
    };
  }

  factory GeoCoordinates.fromMap(Map<String, dynamic> map) {
    return GeoCoordinates(
      longitude: map['Longitude'],
      latitude: map['Latitude'],
      altitude: map['Altitude'],
      timestamp: map['Timestamp'],
    );
  }

  String toJson() => json.encode(toMap());

  factory GeoCoordinates.fromJson(String source) => GeoCoordinates.fromMap(json.decode(source));

  @override
  String toString() {
    return 'GeoLocation(longitude: $longitude, latitude: $latitude, altitude: $altitude, timestamp: $timestamp)';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
  
    return other is GeoCoordinates &&
      other.longitude == longitude &&
      other.latitude == latitude &&
      other.altitude == altitude &&
      other.timestamp == timestamp;
  }

  @override
  int get hashCode {
    return longitude.hashCode ^
      latitude.hashCode ^
      altitude.hashCode ^
      timestamp.hashCode;
  }
}
